@@ -8,8 +8,12 @@
 #	./Configure -des -Dopenbsd_distribution=defined
 #
 
-# OpenBSD has a better malloc than perl...
-test "$usemymalloc" || usemymalloc='n'
+# In OpenBSD > 3.7, use perl's malloc [perl #75742]
+case "$osvers" in
+3.[89]*|[4-9]*)
+    test "$usemymalloc" || usemymalloc=y
+    ;;
+esac
 
 # malloc wrap works
 case "$usemallocwrap" in
@@ -118,72 +122,13 @@ $define|true|[yY]*)
 	;;
 	esac
 	case "$osvers" in
-	[012].*|3.[0-5])
-        	# Broken at least up to OpenBSD 3.5, we'll see about 3.6
+	[012].*|3.[0-6])
+        	# Broken at least up to OpenBSD 3.6, we'll see about 3.7
 		d_getservbyname_r=$undef ;;
 	esac
 esac
 EOCBU
 
-# This script UU/use64bitint.cbu will get 'called-back' by Configure 
-# after it has prompted the user for whether to use 64-bitness.
-cat > UU/use64bitint.cbu <<'EOCBU'
-case "$use64bitint" in
-$define|true|[yY]*)
-	echo " "
-	echo "Checking if your C library has broken 64-bit functions..." >&4
-	$cat >check.c <<EOCP
-#include <stdio.h>
-typedef $uquadtype myULL;
-int main (void)
-{
-    struct {
-	double d;
-	myULL  u;
-    } *p, test[] = {
-	{4294967303.15, 4294967303ULL},
-	{4294967294.2,  4294967294ULL},
-	{4294967295.7,  4294967295ULL},
-	{0.0, 0ULL}
-    };
-    for (p = test; p->u; p++) {
-	myULL x = (myULL)p->d;
-	if (x != p->u) {
-	    printf("buggy\n");
-	    return 0;
-	}
-    }
-    printf("ok\n");
-    return 0;
-}
-EOCP
-	set check
-	if eval $compile_ok; then
-	    libcquad=`./check`
-	    echo "Your C library's 64-bit functions are $libcquad."
-	else
-	    echo "(I can't seem to compile the test program.)"
-	    echo "Assuming that your C library's 64-bit functions are ok."
-	    libcquad="ok"
-	fi
-	$rm -f check.c check
-
-	case "$libcquad" in
-	    buggy*)
-		cat >&4 <<EOM
-
-*** You have a C library with broken 64-bit functions.
-*** 64-bit support does not work reliably in this configuration.
-*** Please rerun Configure without -Duse64bitint and/or -Dusemorebits.
-*** Cannot continue, aborting.
-
-EOM
-		exit 1
-		;;
-	esac
-esac
-EOCBU
-
 # When building in the OpenBSD tree we use different paths
 # This is only part of the story, the rest comes from config.over
 case "$openbsd_distribution" in
